; ---------------------------------------------------------------
; Open IncludeFile - Popup Tool for PB IDE  - compiled w/PB 5.11 
; by horst.schaeffer@gmx.net
; ---------------------------------------------------------------
#title = "InclOpen"
#info = #title + " Version 2.20" + #CRLF$ + "by Horst Schaeffer" +#CRLF$+#CRLF$ +"See Readme.txt"
#break = 0    
#q2$ = Chr(34)
; ---------------------------------------------------------------
PBsourceFile.s = ProgramParameter() 
Global SourceDir.s 
If FileSize(PBsourceFile) > 0 
  SourceDir.s = GetPathPart(PBsourceFile) 
Else 
  MessageRequester(#title,#info,#MB_ICONASTERISK) 
  End   
EndIf 
; ---------------------------------------------------------------
CursorInfo$ = LCase(GetEnvironmentVariable("PB_TOOL_CURSOR"))  ; line x column, eg. 123x45
Repeat 
  parm$ = LCase(ProgramParameter())
  If parm$ = "" : Break : EndIf 
  If parm$ = "/s" : ToSort = #True 
  ElseIf CursorInfo$ = "" : CursorInfo$ = parm$ 
  EndIf 
Until #break 

CursorLine = Val(StringField(CursorInfo$,1,"x"))

; ---------------------------------------------------------------
;- List of files 
; ---------------------------------------------------------------
Structure ipath : path.s : group.l : EndStructure 
NewList item.ipath()

Macro AddToFileList(fgroup,fpath)   ; add fname w/path, sort by group, name 
  If ToSort 
    Lname.s = LCase(GetFilePart(fpath))   ; fname,lower case 
    If LastElement(item())
      Repeat 
        If fgroup < item()\group 
        ElseIf fgroup = item()\group And Lname < LCase(GetFilePart(item()\path))
        Else: Break
        EndIf 
        If PreviousElement(item()) = #NUL : ResetList(item()) : Break : EndIf 
      Until #break 
    EndIf 
  EndIf 
  AddElement(item())
  item()\group = fgroup
  item()\path = fpath
EndMacro

; ---------------------------------------------------------------
;- Scan source file 
; ---------------------------------------------------------------
IDEpath.s = GetEnvironmentVariable("PB_TOOL_IDE")  ; PureBasic.exe with full path
Global CompilerHome.s = GetPathPart(IDEpath)       ; directory path
Global Pathconstant.s  ; only supports #IncludePath

Procedure.s ExtractPath(string.s)   ; assume comments and secondary statements (:) CUT OFF 
  string = ReplaceString(string,"#includepath",Pathconstant,#PB_String_NoCase)
  string = ReplaceString(string,"#pb_compiler_home",CompilerHome,#PB_String_NoCase)
  string = ReplaceString(string,"#pb_compiler_filepath",SourceDir,#PB_String_NoCase)
  i.i : Path.s
  Repeat                                         ; assemble path assignment 
    i +1
    part$ = Trim(StringField(string,i,"+"))      ; parts concatenated with "+" 
    If Asc(part$) = '"'
      part$ = StringField(part$,2,#Q2$)          ; strip quote marks and whatever beyond (colon..) 
    EndIf 
    Path + part$  
  Until part$ = ""
ProcedureReturn Path
EndProcedure 

If ReadFile(0,PBsourceFile) = #NUL : End : EndIf   ; oops 
BOM = ReadStringFormat(0)
IncludeFileDir.s = SourceDir                       ; default 

While Eof(0) = #False                              ; scan for "Includefile" and related
  Line.s = LTrim(ReadString(0,BOM)) : LineNumber +1
  line = StringField(line,1,";")                   ; assuming no semicolons within quote marks
  KeyWord.s = LCase(StringField(Line,1," "))       ; up to blank

  If KeyWord = "#includepath"                      ; constant, must be succeeded by blank space
    Pathconstant = Trim(StringField(Line,2,"="))   ; assignment
  Else 
    RestOfLine.s = Mid(Line,Len(KeyWord)+1)
    
    If Keyword = "includepath"
      path.s = ExtractPath(RestOfLine)                           ; also removes quote marks
      If path = ".\" : path = "" : EndIf 
      If path And Right(path,1) <> "\" :  path + "\" :  EndIf    ; add backslash
      If Mid(Path,2,1) <> ":" : Path = SourceDir + Path : EndIf  ; relative path? 
      IncludeFileDir = Path                                      ; take it for following IncludeFiles
      
    ElseIf Keyword = "includefile" Or Keyword = "xincludefile"
      path = ExtractPath(RestOfLine)
      If path  
        If Mid(path,2,1) <> ":" : path = IncludeFileDir + path : EndIf 
        If LCase(Left(path,Len(SourceDir))) = LCase(SourceDir) : group = 0 
        Else : group = 1  ; not in main source dir environment
        EndIf 
        AddToFileList(group,path)
        If LineNumber = CursorLine : ItemPicked = #True : Break : EndIf    ; immediate pick 
      EndIf 
    EndIf
  EndIf 
Wend 
CloseFile(0)

; ---------------------------------------------------------------
;- Main 
; ---------------------------------------------------------------

If ItemPicked = #False         ; offer menu 
  If ListSize(item())
    Window = OpenWindow(0,0,0,1,1,#title,#PB_Window_Invisible) 
  EndIf 
  If Window 
    CreatePopupMenu(0) 
    ForEach item() 
      If item()\group > 0 And index > 0 And bar = 0
        MenuBar() : bar = 1 
      EndIf 
      MenuItem(index,GetFilePart(item()\path)) : index + 1
    Next  
    DisplayPopupMenu(0,Window) 

    AddWindowTimer(0,1234,100)    ; suspended until memu closed/aborted  
    Repeat 
      event = WaitWindowEvent()
      If event = #PB_Event_Menu
        ItemPicked = SelectElement(item(),EventMenu())
      ElseIf event = #PB_Event_Timer 
        exit = #True  
      EndIf 
    Until ItemPicked|exit  
    RemoveWindowTimer(0,1234)
  EndIf 
EndIf 

If ItemPicked 
  ShellExecute_(Window,"open",IDEpath,#q2$+item()\path+#q2$,"",#SW_SHOW)
EndIf  
End


; IDE Options = PureBasic 5.30 (Windows - x86)
; CursorPosition = 5
; Folding = -
; EnableUnicode
; Executable = InclOpen.exe
; CommandLine = "F:\Programming\PureBasic\Projects\Pb\InclOpen\InclOpen.pb"
; CompileSourceDirectory